@@ -17,7 +17,8 @@ pr int wert, pos = 0;
                     * array elements in a[] to be sorted.  */
 sh int a[] = { 5, 7, 3, 2, 17, 9, 4, 23, 0, 11, 19, 8, 14, 33, 2, 38 };
 
-main(){
+void main( void )
+{
   start { 
     sh int gewicht;
     farm assert ( __STARTED_PROCS__ <= Nmax );
