@@ -170,7 +170,7 @@ sync cplx *FFT( sh cplx *a, sh int n, sh cplx w )
 }
 
 
-main()
+void main( void )
 {
  start {
 #ifdef LARGE_N
