@@ -287,7 +287,7 @@ extern sh int _ticket[_MAXJOINS];  /* semaphore to control entry of passengers *
             asm("pshgc\t sps"); /* preset sync cell */ \
             /* now the bus is ready for departure. Close the door: */ \
             _delaystmt; /*wait*/ ; \
-            write(1,"gone!\n",6); \
+            /*write(1,"gone!\n",6);*/ \
             _gone[_joinID] = 1; \
             asm("nop\nnop\nnop\nnop\nnop\nnop\nnop"); \
             asm("gethi\t __ticket,r30"); /* r30 <- _ticket[_joinID]:*/ \
@@ -330,7 +330,7 @@ extern sh int _ticket[_MAXJOINS];  /* semaphore to control entry of passengers *
             asm("getlo\t -1,r31"); \
             asm("mpadd\t gps,1,r31"); \
             $ = _old_dollar;    /* restore old $ */ \
-            write(1,"thrown out\n",10); \
+            /*write(1,"thrown out\n",10);*/ \
             /* now remove the frames and reset the stack pointers: */ \
             asm("ldg\t gpp,0,gpp"); \
             asm("mov\t gpp,spp"); \
