@@ -123,6 +123,7 @@ extern int unmygetchar( int c, char *stdin_buffer );
 
 
 #define	_NFILE	20	/* initial number of streams: SPARC ABI and default */
+extern int _filepos[_NFILE]; /* store current (physical) r/w position in file */
 #define FILEALLOC malloc   /* allocation routine for generating FILE ptrs */
 
 #define	_IOFBF		0000	/* full buffered */
@@ -185,13 +186,17 @@ extern char	*gets(char *);
 /*extern int	putc(int, FILE *);*/
 #define putc(c,fp) fputc(c,fp)
 
-/* DIRECT FILE ACCESS NOT YET SUYPPORTED, SORRY
+extern int	fseek(FILE *, long, int);
+extern long	ftell(FILE *);
+#define	SEEK_SET	0
+#define	SEEK_CUR	1
+#define	SEEK_END	2
+
+/* OTHER DIRECT FILE ACCESS NOT YET SUPPORTED, SORRY
 extern size_t	fread(void *, size_t, size_t, FILE *);
 extern size_t	fwrite(const void *, size_t, size_t, FILE *);
 extern int	fgetpos(FILE *, fpos_t *);
-extern int	fseek(FILE *, long, int);
 extern int	fsetpos(FILE *, const fpos_t *);
-extern long	ftell(FILE *);
 extern void	rewind(FILE *);
 */
 
