@@ -22,7 +22,7 @@ extern double sqrt( float );    /* see forklib2.asm */
  * with sqrt(p/2) < isqrt(p) < sqrt(p).
  */
 
-#define pow(x,y) 1.0  /* eliminate linker complaints. to be implemented later CWK*/
+/*#define pow(x,y) 1.0  /* eliminate linker complaints. to be implemented later CWK*/
 
 /* Routines located in math.c: */
 
@@ -54,4 +54,5 @@ extern float  fabs( float  );
 
 extern int abs( int );
 
+extern float pow( float, float );
 #endif
