@@ -70,23 +70,31 @@ void rw_lockup( rw_lock *lock, int ptype)
        /* writer has finished - now try to catch lock */
        if( mpadd(&(lock->readercounter), 1) & __RW_FLAG__ ) {
           /* another writer was faster than me */
-          wait = mpadd( &(lock->readercounter), -1); /* undo lock; wait as dummy */
+          mpadd( &(lock->readercounter), -1); 
+          /* undo lock */
           wait = 1; /* wait = TRUE */
        }
     }
  }
  else {      /* writer-lock */
     /* wait for writer to leave the lock */
+#if 0
     unsigned int rc = lock->readercounter;
+#endif
     fair_lockup(&(lock->writerlock));
     /*  set flag to indicate to the readers that I want to have the lock */
+#if 0
     asm("bmc    0\n\
          gethi  0x40000000,r31 /*__RW_FLAG__*/\n\
          syncor r31,%rc\n\
          nop    /*delay*/");
          /*\n\ nop    \n\ stg    r31,r30,0 */
+#endif
+    syncor( (int *)&(lock->readercounter), __RW_FLAG__ );
     /* wait for readers to leave the lock */
     while( mpadd( &(lock->readercounter), 0) & (__RW_FLAG__-1));
+    /*     ^^^^^ Zuweisung an Dummy koennte Probleme machen. -> asm */
+    /*     evtl. benutze syncor()-Routine */
  }
 } /* _rw_lockup() */
 
