@@ -42,8 +42,6 @@ extern int errno;
 
 #include "shmalloc.c"
 
-#include "graphic.c"
-
 #if 0
 /* This stuff is not usable for FORK.    CWK */
 
@@ -777,7 +775,7 @@ double strtod (const char *nptr, char **endptr)
 	goto overflow;
     }
 #endif
-  pprintf("%d\n", itento(-exponent));
+  /*pprintf("%d\n", itento(-exponent));*/
   if (exponent >0) dnum = (itof(num)) * (float)itento(exponent);
   if (exponent <0) dnum = (itof(num)) / (float)itento(-exponent);
   return dnum * itof(sign);
