@@ -21,7 +21,8 @@ int compare( void *A, void *B)
 }
 
 
-main() {
+void main( void ) 
+{
  pr int i;
  start {
    sh int *a;
