@@ -19,6 +19,7 @@ sh int *belongs_to_hull;
                   indicates whether a point belongs to the hull*/
 sh simple_lock screen = 0;
 #define PIXELS 512      /*extents of generated X-Pixmap picture*/
+sh double factor = 0.00000024;  /*fuer 512*/
 
 /* output routines: */
 
@@ -77,10 +78,13 @@ sync int *delete_right( sh int *pt, sh int *num, sh int p1, sh int p2 )
  sh int *left;
  sh int n = *num;
  sh int p = 0;
+ 
  $ = mpadd( &p, 1 );   /* renumber and determine number of processors */
 
+ farm pprintf("delete_right P%d,P%d, n=%d\n", p1, p2, n );
+
  /* delete all points in pt located right from line p1p2: */
- seq  left = (int *) shmalloc( n );
+ farm if ($==0) left = (int *) shmalloc( n );
  left[0]=p1; left[1]=p2;
  leftcnt = 2;
  for (j=$; j<n; j+=p)
@@ -167,19 +171,19 @@ sync void seq_qh( sh int *pt, sh int n )
 
  /* DC step: select any pivot point from pt. We have p1==pt[0],p2==pt[1]*/
  if (n==2) {
-    seq seq_line (x[p1],y[p1],x[p2],y[p2],7,2);
+    seq seq_line (x[p1],y[p1],x[p2],y[p2],0x00ff00,2);
     return;
  }
  if (n==3) {  /* one point (beyond old p1,p2) must belong to hull */
     belongs_to_hull[pt[2]] = 1;      /* saves a recursive call */
-    seq seq_line (x[p1],y[p1],x[pt[2]],y[pt[2]],7,2);
-    seq seq_line (x[pt[2]],y[pt[2]],x[p2],y[p2],7,2);
+    seq seq_line (x[p1],y[p1],x[pt[2]],y[pt[2]],0xff00ff,2);
+    seq seq_line (x[pt[2]],y[pt[2]],x[p2],y[p2],0xff00ff,2);
     return;
  }
  pivot = seq_pivotize( pt, n );
  belongs_to_hull[pivot] = 1;
- seq seq_line (x[p1],y[p1],x[pivot],y[pivot],6,2);
- seq seq_line (x[pivot],y[pivot],x[p2],y[p2],6,2);
+ seq seq_line (x[p1],y[p1],x[pivot],y[pivot],0xff00ff,2);
+ seq seq_line (x[pivot],y[pivot],x[p2],y[p2],0xff00ff,2);
  leftcnt1 = n;
  left1 = delete_right( pt, &leftcnt1, p1, pivot );
  seq_qh( left1, leftcnt1 );
@@ -198,15 +202,17 @@ sync void qh( sh int *pt, sh int n )
  sh int *S1, *S2, n1, n2;     /* to hold subproblems */
  sh int firstrightproc;
 
+ farm pprintf("qh(%d)\n", n);
+
  $ = mpadd( &p, 1 );   /*renumber $ and determine p*/
  if (n==2) {
-    line (x[p1],y[p1],x[p2],y[p2],7,2);  /* draw hull edge */
+    line (x[p1],y[p1],x[p2],y[p2],0xff,2);  /* draw hull edge */
     return;
  }
  if (n==3) {  /* one point (beyond old p1,p2) must belong to hull */
     belongs_to_hull[pt[2]] = 1;
-    line (x[p1],y[p1],x[pt[2]],y[pt[2]],7,2); /* draw hull edge */
-    line (x[pt[2]],y[pt[2]],x[p2],y[p2],7,2); /* draw hull edge */
+    line (x[p1],y[p1],x[pt[2]],y[pt[2]],0xff00ff,2); /* draw hull edge */
+    line (x[pt[2]],y[pt[2]],x[p2],y[p2],0xff00ff,2); /* draw hull edge */
     return;
  }
  if (p==1) { seq_qh( pt, n ); return; }
@@ -217,8 +223,8 @@ sync void qh( sh int *pt, sh int n )
  if (n>p+2)  pivot = par_pivotize( pt, n );
  else        pivot = seq_pivotize( pt, n );
  belongs_to_hull[pivot] = 1;
- line (x[p1],y[p1],x[pivot],y[pivot],6,2); /* draw edge (p1,pivot) */
- line (x[pivot],y[pivot],x[p2],y[p2],6,2); /* draw edge (pivot,p2) */
+ line (x[p1],y[p1],x[pivot],y[pivot],0x00ff00,2); /* draw edge (p1,pivot) */
+ line (x[pivot],y[pivot],x[p2],y[p2],0x00ff00,2); /* draw edge (pivot,p2) */
  /* determine subproblems: */
  n1 = n2 = n;
  S1 = delete_right( pt, &n1, p1, pivot );
@@ -233,15 +239,10 @@ sync void qh( sh int *pt, sh int n )
 }
 
 
-sh double factor = 0.00000024;
 sh unsigned int starttime, stoptime;
 
-main()
+void main( void )
 {
- start {
-    init_pict( PIXELS+1, PIXELS+1 );
-    clear_pixels( 0 );
- }
  if (__PROC_NR__==0) {
     printf("Enter N = ");
     scanf("%d", &N );
@@ -257,11 +258,15 @@ main()
    sh int uppercnt, lowercnt;
    pr int j;
    sh double xj;
-   farm belongs_to_hull = (int *) shmalloc( N );
-   seq {
-     x = (double *) shmalloc( N * sizeof(double) );
-     y = (double *) shmalloc( N * sizeof(double) );
+
+   init_pict( PIXELS+1, PIXELS+1 );
+   clear_pixels( 1 );
+
+   belongs_to_hull = (int *) shalloc( N );
+   x = (double *) shalloc( N * sizeof(double) );
+   y = (double *) shalloc( N * sizeof(double) );
       
+   seq {
      /* sequential to get equal problems for varying p */
      for (j=0; j<N; j++) {
 #define ABS(x) (((x)>=0)?(x):(-(x)))
@@ -285,7 +290,7 @@ main()
    uppercnt = lowercnt = N;
    upper = delete_right( pt, &uppercnt, minxpt, maxxpt );
    lower = delete_right( pt, &lowercnt, maxxpt, minxpt );
-   line (x[minxpt],y[minxpt],x[maxxpt],y[maxxpt],4,2);
+   line (x[minxpt],y[minxpt],x[maxxpt],y[maxxpt],0xffff,2);
    qh( upper, uppercnt );
    qh( lower, lowercnt );
 
