@@ -177,8 +177,16 @@ void gencode(caller, callee) Symbol caller[], callee[]; {
    cp->prev = codelist;
    for (bp = 0, cp = &codehead; errcnt <= 0 && cp; cp = cp->next)
       switch (cp->kind) {
-      case Start: case Asm: case Switch: case Pcode:   /*CWK*/
+      case Start: case Asm: case Switch: 
          break;
+      case Pcode:   /*CWK*/
+         { extern int resetshoffset;
+          assert(cp->u.node);        /*970903*/
+          if (cp->u.node->resetshoffset) {
+             resetshoffset = 1;
+          }
+          break;
+         }
       case Defpoint: 
          src = cp->u.point.src;
          break;
@@ -260,6 +268,7 @@ static Node list(p) Node p; {
 Node listnodes(tp, tlab, flab) Tree tp; {
    Opcode op;
    Node p, l, r;
+   extern int shframedepth;  /*970902*/
 
    if (tp == 0)
       return 0;
@@ -597,6 +606,8 @@ Node listnodes(tp, tlab, flab) Tree tp; {
       p = node(tp->op, 0, 0, tp->u.sym);
       p->synchron = tp->synchron;      /*durchreichen CWK*/
       p->maybe_priv = 0;    /*CWK*/
+      assert(shframedepth>=0);
+      p->shframedepth = shframedepth;   /*970902*/
       break;
    case ADDRSG: case ADDRSF:                /*CWK, vorlaeufig, nur kopiert */
       assert(tlab == 0 && flab == 0);
@@ -613,14 +624,20 @@ Node listnodes(tp, tlab, flab) Tree tp; {
       p->synchron = tp->synchron;      /*durchreichen CWK*/
       p->maybe_priv = 1;    /*CWK*/
       break;
+   case ENGRP: case ENLGRP: case NEXTCASE: 
    case ADR_PROZ_NUM: case ADR_GRP_NUM:
-   case SYNC:
-   case SPLIT: case NEXTCASE: case LGRPIF:
-   case ENGRP: case EXGRP: case XPROC: case RETFLUSH:
-   case ENLGRP: case EXLGRP: case SIZE: case IBP: case FLUSHFRAMES:
+   case SYNC: case SPLIT: case LGRPIF:
+   case EXGRP: case XPROC: case RETFLUSH:
+   case EXLGRP: case SIZE: case IBP: case FLUSHFRAMES:
    case BEGSEQ: case ENDSEQ:
+   case SHSPACE: /*970902*/
       p = newnode(tp->op, 0, 0, 0);
       p->synchron = tp->synchron;      /*durchreichen CWK*/
+      p->resetshoffset = 0; /*970903*/
+      if (op==ENGRP || op==SPROC /* nicht auskommentiert 970908*/
+          || op==ENLGRP || op==NEXTCASE) {
+          p->resetshoffset = 1;        /*970903*/
+      } 
       break;
    case MKGRP: case SPROC: case ILOG2:
       assert(tlab == 0 && flab == 0);
