@@ -132,9 +132,10 @@ static Tree call(f, fty, src) Tree f; Type fty; Coordinate src; {
      if (isptr(f->type) && isfunc(f->type->type)) {
        Symbol symb; /*950522 =  f->kids[0]->u.sym;   /*CWK*/
        Tree myf = f->kids[0];  /*CWK 950522*/
+       assert(myf);  /*970811*/
        while (myf && !myf->u.sym)  myf = myf->kids[0];   /*950522*/
        symb = myf->u.sym;
-       /*assert(symb); oben eingebaut in Abfrage 950522*/
+       assert(symb); /*970811*/ /* oben eingebaut in Abfrage 950522*/
        if (symb) {
          sharedpar = symb->sharedparameters;    /*CWK*/
          regpar = symb->regparameters;    /*CWK*/
