@@ -26,8 +26,10 @@ static Node lhead;
 
 static int rflag;      /* != 0 to trace register allocation */
 static int shframesize, prframesize;      /* CWK: size of sh/pr activation record */
+       int shframedepth;  /* CWK 970902, used in dag.c*/
 static int proffset;      /* CWK: vormals offset, current private frame offset */
-static int shoffset;      /* CWK: current shared frame offset */
+       int shoffset;      /* CWK: current shared frame offset */
+       int resetshoffset=0; /*970903: flag indicates that a new group has just started*/
 static int argbuildsize;   /* size of argument build area */
 static int argoffset;      /* offset from top of stack for next argument */
 static int nregs = FREE_REGS;   /* number of allocatable registers */
@@ -88,7 +90,11 @@ void address(Symbol q, Symbol p, int n) {
           *Register uebergeben worden sein kann. Verarztung in ADDRxxx CWK*/
       }
       else {                          /* lokale Variable */
-        q->x.name = stringf("%s,%d", (p->shared)?"fps":"fpp", q->x.offset); /*CWK*/
+        q->x.name = stringf("%s,%d/*via address*/", (p->shared)?"fps":"fpp", 
+          (p->shared)? 2+q->x.offset : q->x.offset ); /*CWK 970908*/
+/* Achtung!
+ * in asmcode muss hier auf sh locals verzichtet werden.
+ */
       }
    }
 }
@@ -109,6 +115,13 @@ void blockbeg(Env *e) {
    e->rmask = rmask;
    e->proffset = proffset;  /* statt offset CWK */
    e->shoffset = shoffset; /*CWK*/
+   assert(shframedepth>=0);
+   e->framedepth = shframedepth;  /*970902*/
+   if (resetshoffset) {
+       print("/*RESET shoffset, vorher %d*/\n",shoffset);
+       shoffset = 0; /*970903*/
+       resetshoffset = 0;
+   }
 }
 
 /* blockend - end a compound statement */
@@ -117,6 +130,7 @@ void blockend(Env *e) {
    if (shoffset > shframesize)   shframesize = shoffset;  /*CWK*/
    proffset = e->proffset;
    shoffset = e->shoffset;
+   shframedepth = e->framedepth;  /*970902*/
    rmask = e->rmask;
 }
 
@@ -620,7 +634,28 @@ void emit(Node p) {
          print("add\t r%d,(%s)&0x1fff,r%d  /*dto*/\n", r, p->syms[0]->x.name, r);
          break;
       case ADDRSLP:       /*CWK*/
-         print("add\t %s,r%d\t /*ADDRSLP*/\n", p->syms[0]->x.name, r);
+#if 0
+         print("add\t %s,r%d\t /*ADDRSLP*/\n", p->syms[0]->x.name, r);/*vorher*/
+#endif
+         /* 970903*/
+         if (p->syms[0]->x.framedepth == 0)  /* access at top level of fn. */
+           if (!cfunc->sync)  /*980129: start bastelt einen fake-fps!*/
+             print("add\t fps,%d,r%d\t /*ADDRSLP a*/\n", 1+p->syms[0]->x.offset, r);
+           else
+             print("add\t fps,%d,r%d\t /*ADDRSLP b*/\n", 1+p->syms[0]->x.offset, r);
+               /*vorher 2 statt 1 ^ da Fn-Grfr. dazw., jetzt retval weg */
+         else   /*framedepth > 0:*/
+         if (p->shframedepth - p->syms[0]->x.framedepth >= 1) {  /*new 970902*/
+           int j;
+           print("ldg gps,0,r31  /*ADDRSLP, nested(%d) [%d]*/\n nop\n",
+             p->shframedepth - p->syms[0]->x.framedepth,
+                               p->syms[0]->x.framedepth ); 
+           for ( j=p->shframedepth - p->syms[0]->x.framedepth - 1; j>0; j--)
+             print("ldg r31,0,r31  /*ADDRSLP, nested(%d)*/\n nop\n", j);
+           print("add\t r31,%d,r%d\t /*ADDRSLP c*/\n", 1 + p->syms[0]->x.offset, r);
+         }                                        /* ^ = gps alt + sync cell (spaeter + @) */
+         else  /*declared by same group:*/
+           print("add\t gps,%d,r%d\t /*ADDRSLP d*/\n", 1 + p->syms[0]->x.offset, r);
          break;
       case ADDRSFP:
          print("add\t %s,r%d\t /*ADDRSFP*/\n", p->syms[0]->x.name, r);  /*CWK*/
@@ -633,6 +668,13 @@ void emit(Node p) {
          print("bsrg\t spp,forklib_sync\t /*SYNCV*/\n");   /**/
          break;
 
+     case SHSPACEV:                       /*970902*/
+         /* allocate space for shared block-local variables */
+         if (Pcodepar > 0)
+            print("add\t sps,%d,sps\t /*alloc. stack space for sh locals*/\n",
+                   Pcodepar );
+         break;
+
       case SPLITV:                /*CWK: berechne Trennzelle im SM*/
          force_modulo_0;
          print("sub\t sps,eps,Ret\t /*split*/\n");
@@ -810,6 +852,7 @@ void emit(Node p) {
          ausk. 950815 CWK: Modulierung nur fuer 1. Befehl erforderlich
          print("mov\t gps,sps\t /*exgrp: free frame*/\n");        /*1*/
 #endif
+
          print("ldg\t gpp,0,gpp\t /*exgrp: old gpp*/\n");
          print("add\t gpp,-2,spp\t /*exgrp: free frame*/\n");
          print("popgn\t spp,sps\t /*exgrp: old sps*/\n");
@@ -915,6 +958,7 @@ void emit(Node p) {
             print("ldg\t par2,0,par2\t /*sproc: $orig.*/\n");       /*0*/
          }
          /* SHADOW wurde behandelt. Nun erzeuge neuen privaten Gruppenframe: */
+         print("pshg\t fps,spp\t /*sproc*/\n"); /*980129*/
          print("pshg\t eps,spp\t /*sproc*/\n"); 
          print("pshg\t sps,spp\t /*sproc*/\n");
          print("pshg\t gpp,spp\t /*sproc*/\n");
@@ -945,13 +989,15 @@ void emit(Node p) {
          print("pshgc\t sps    \t /*sproc: init synccell*/\n");
          print("getlo\t 1,r30\n");
          print("mpaddn\t gps,1,r30\t /*sproc: set synccell*/\n");
+         print("mov\t gps,fps\t /*sproc: fake fps for ADDRSL*/\n");/*980129*/
          break;
 
       case XPROCV:                /*CWK*/
          /*!alle Aenderungen hier muessen auch nach retflush() uebertragen werden!*/
          /*Die Gruppenrahmen vor der start-Anweisung werden wiederhergestellt.*/
-         force_modulo_0; /*_1*/
+         force_modulo_1;
          /*print("bsrg\t spp,forklib_sync\n"); weg nach Absprache mit HS */
+         print("ldg\t gpp,-3,fps\t /*xproc: restore fps*/\n"); /*1*980129*/
          print("ldg\t gps,0,gps\t /*xproc: restore gps*/\n");       /*0*/
          print("mov\t gpp,spp\t /*xproc: prepare spp*/\n");
          print("ldg\t gpp,0,gpp\t /*xproc: restore gpp*/\n");       /*0*/
@@ -970,7 +1016,10 @@ CWK 960812
             print("ibp4\n");
          }
          /*Alle Prozessoren ausser 0 setzen das SHADOW-Bit:*/
+#if 0
+    970904: Wieso ??? Dabei kann ich mir den Prozedurframe zerhacken!
          print("add\t gpp,3,spp\t /*xproc: restore spp*/\n");
+#endif
 #if 0
 CWK 960812   SHADOW wird ganz eliminiert.
          print("ldg\t par2,0,par2\t /*xproc: $orig.*/\n");           /*0*/
@@ -1021,11 +1070,16 @@ CWK 960812   SHADOW wird ganz eliminiert.
          print("mpadd\t gps,1,r31\t /*exlgrp:check out*/\n");           /*1*/
 #endif
          print("ldg\t gps,0,gps\t /*exlgrp: rest.gps*/\n");             /*0*/
+/*testweise drinlassen: 970905*/
          print("add\t sps,-2,sps\t /*exlgrp: free frame*/\n");           /*1*/
          /*vormals add gps,2,sps fuehrt bei dazwischenliegendem Prozedurframe
           *zum Chaos. Da der sps von Schleifen nicht zwischengespeichert wird,
           *muss der sps mit dem Holzhammer korrigiert werden: sps-=2.
           *Dies macht nix, weil das SM bei loops nicht aufgesplittet wird. CWK*/
+#if 0
+ so muesste es eigentlich sein am 970905
+         print("mov\t gps,sps\t /*exlgrp: free group frame*/\n");           /*1*/
+#endif
          if (glevel)  /*950504*/
             print("ibp4\n");
 #ifdef DEBUG_EXLGRPV
@@ -1083,10 +1137,9 @@ CWK 960812   SHADOW wird ganz eliminiert.
             print("ldg\t gpp,-4,eps\t /*FLUSHFRAME(%d of %d)*/\n", label1, Pcodepar);
             print("getlo\t -1,r31   \t /*FLUSHFRAME(%d of %d)*/\n", label1, Pcodepar);
             print("ldg\t gpp,0,gpp\t /*FLUSHFRAME(%d of %d)*/\n", label1, Pcodepar);
-            modulo_nop;
+            print("mov\t gps,sps\t /*FLUSHFRAME(%d of %d)*/\n", label1, Pcodepar);
             print("ldg\t gps,0,gps\t /*FLUSHFRAME(%d of %d)*/\n", label1, Pcodepar);
             print("add\t gpp,3,spp\t /*FLUSHFRAME(%d of %d)*/\n", label1, Pcodepar);
-            print("add\t gps,2,sps\t /*FLUSHFRAME(%d of %d)*/\n", label1, Pcodepar);
             /* in der naechstunteren Gruppe austragen: */
             print("mpaddn\t gps,1,r31\t /*FLUSHFRAME(%d of %d)*/\n",label1,Pcodepar);
             if (glevel) { /*950504*/
@@ -1129,6 +1182,8 @@ void function(Symbol f, Symbol caller[],
 
    proffset = 1;  /*statt 4 CWK*/
    shoffset = 1;  /*CWK*/
+   resetshoffset = 0;
+   shframedepth = 0;  /*970902*/
    freeallparregs();  /*CWK, fuer alle Faelle*/
    for (i = 0; caller[i] && callee[i]; i++) {
       if (callee[i]->shared) {   /*CWK*/
@@ -1145,8 +1200,10 @@ void function(Symbol f, Symbol caller[],
       }
       callee[i]->sclass = AUTO;
    }
-   usedmask = argbuildsize = prframesize = shframesize = shoffset = proffset = 0; /*CWK*/
+   usedmask = argbuildsize = prframesize = shframesize
+            = shoffset = proffset = 0; /*CWK*/
    gencode(caller, callee);
+   assert(shframedepth==0);
    /*print("%s:.word 0x%x\n", f->x.name, usedmask&~0x3f); CWK: .word... weg*/
    print("%s:\n", f->x.name);  /*CWK*/
    /* pc wurde gerade durch call gerettet. Jetzt fpp und sr retten: CWK*/ 
@@ -1165,16 +1222,26 @@ void function(Symbol f, Symbol caller[],
      print("pshg\t eps,spp\t /*save old eps*/\n");   /*940919*/      /*CWK, 1*/
    }
    else { /*async*/
+#if 0
+     versuchsweise auskommentiert 970905
+     versuchsweise wieder drin wegen ADDRSL-Problem 980129
+#endif
      if (shframesize) {   /*shared proc frame for async fn:*/
         print("pshg\t fps,sps\t /*save old fps*/\n");   /*950331*/   /*CWK, 1*/
         print("add\t sps,-1,fps\t /*new fps*/\n");  /*950331*/
      }
      print("mov\t spp,fpp\t /*set new fpp*/\n");   /*CWK,                   0*/
    }
-#ifdef SAVE_STATUS_REGISTER
-   modulo_nop;   und den fpp korrigieren
-   print("pshg\t sr,spp\t\t /*save statusreg*/\n");        /*CWK          1*/
-#endif
+
+   if (f->sync) {  /* Funktionsgruppe aufmachen: 950111 Vor Reservierung
+                     fuer lokale Variablen gezogen 970905 */
+      force_modulo_0;
+      print("ldg\t gps,1,r31\t /*Funktionsgruppe aufmachen: lade Prozessorzahl*/\n");
+      print("pshg\t gps,sps\t /*Funktionsgruppe: rette alten gps*/\n");
+      print("add\t sps,-1,gps\t /*Funktionsgruppe: neuer gps*/\n");
+      print("pshg\t r31,sps\t /*Funktionsgruppe: init s.cell*/\n");
+   }
+
    /* framesize += (4*) nregs + argbuildsize; */
    /*CWK: Nach Neuorganisation des Stackframes brauchen wir die nregs und die
     * argbuildsize nicht mehr. Wenig sinnvoll, weil als naechster Frame auch 
@@ -1182,17 +1249,20 @@ void function(Symbol f, Symbol caller[],
    /*print("subl2 $%d,sp\n", framesize);     CWK*/
    print("add\t fpp,%d,spp\t /*alloc space f locals*/\n", prframesize + 1);/*CWK,0*/
    /* +1 wegen zusaetzlicher Reservezelle, auf die der fpp zeigt*/
-   if (f->sync || shframesize)   /*950331*/
-     print("add\t fps,%d,sps\t /*alloc space f locals*/\n", shframesize + 1);/*CWK,1*/
-   /* nun zeigt spp/sps auf 1. freie Stackzelle nach letztem local. CWK */
+   if (f->sync)
+#if 0
+ offensichtlich braucht man das shframesize gar nicht mehr...
+ versuchsweise raus 970908
+     if (shframesize)   /*950331*/  /*970905*/
+       print("add\t fps,%d,sps\t /*alloc space f locals*/\n",
+             2 + shframesize + 1);
+          /* 2 fuer Funktionsgruppe */
+     else
+#endif
+     /*neu: da sh lokale Var. relativ zur Funktionsgruppe, bleibt sps=fps+1+2 */
+       print("add\t fps,2,sps\t /*space for fn group frame */\n");
+/* vor 980205 war's hier^ eine 3 fuer struct returnvalue */
   
-   if (f->sync) {  /* Funktionsgruppe aufmachen: 950111 */
-      force_modulo_0;
-      print("ldg\t gps,1,r31\t /*Funktionsgruppe aufmachen: lade Prozessorzahl*/\n");
-      print("pshg\t gps,sps\t /*Funktionsgruppe: rette alten gps*/\n");
-      print("add\t sps,-1,gps\t /*Funktionsgruppe: neuer gps*/\n");
-      print("pshg\t r31,sps\t /*Funktionsgruppe: init s.cell*/\n");
-   }
    if (glevel)      /*950504*/
       if (f->sync)
          print("nop\nibp9_0      /* synchronous fn, frame completed */\n");
@@ -1226,7 +1296,11 @@ void Epilog(sync)   /*CWK: alten privaten Stackframe wiederherstellen */
      print("getlo\t -1,r31   \t /*Fn-Gruppe zumachen:*/\n");
      print("mpaddn\t gps,1,r31\t /*austragen,*/\n");
      print("ldg\t gps,0,gps\t /*restore gps,*/\n"); /*0*/
+#if 0
      print("add\t sps,-2,sps\t /*Fn-grp: free frame*/\n");           /*1*/
+#endif
+     /*970904: shared group frame is variable in size:*/
+     print("mov\t gps,sps\t /*Fn-grp: free frame*/\n");           /*1*/
    }
    print("mov\t fpp,spp\t /*free locals*/\n");                 /*1*/
    /*force_modulo_0; auskommentiert, weil nur private Stackoperationen*/
@@ -1241,20 +1315,26 @@ void Epilog(sync)   /*CWK: alten privaten Stackframe wiederherstellen */
      shalloc()-Objekte mehr an. Restaurieren nur explizit mit shallfree().
      print("popg\t spp,eps\t /*alten eps holen*/\n");            /*0*/
      print("popg\t spp,gps\t /*alten gps holen*/\n");            /*0*/
-#endif
-     /*modulo_nop;*/
+     /*ersetzt durch:*/
      print("popng\t spp,-2,gps\t /*restore old gps, ignore eps*/\n");/*0*/
+
+     /*970904 auch auskommentiert:*/
+     print("popng\t gps,0,gps\t /*restore old gps, ignore eps*/\n");/*0*/
      /*     ^-- 960815 */
-     /*modulo_nop;*/
-     print("popg\t spp,gpp\t /*alten gpp holen*/\n");            /*0*/
-     print("mov\t fps,sps\t /*free locals*/\n");                 /*1*/
-     print("popg\t spp,fps\t /*alten fps holen*/\n");            /*0*/
-     /*modulo_nop;*/
+#endif
+     print("add\t spp,-1,spp\t /*ignore old eps*/\n");   /*970904*/
+     print("popg\t spp,gps\t /*restore old gps*/\n"); 
+     print("popg\t spp,gpp\t /*restore old gpp*/\n");
+     print("mov\t fps,sps\t /*free locals*/\n");
+     print("popg\t spp,fps\t /*restore old fps*/\n");
    }
+#if 0
+   versuchsweise auskommentiert 970905
    else if (shframesize) {   /*950331*/
      print("mov\t fps,sps\t /*free locals*/\n");                 /*1*/
      print("popg\t sps,fps\t /*alten fps holen*/\n");            /*0*/
    }
+#endif
    print("popg\t spp,fpp\t /*alten fpp holen*/\n");            /*0*/
    /*Es folgt ret, das den pc poppt und spp nochmals dekrementiert*/
    /*Das Restaurieren des app und aps erfolgt durch den caller. */
@@ -1361,6 +1441,7 @@ static Node *linearize(Node p, Node *last, Node next) {
 void local(Symbol p) {
   if (p->shared) {  /*CWK*/
      p->x.offset = shoffset + 1;
+     p->x.framedepth = p->shframedepth;  /*970902*/
      p->x.name = stringf("fps,%d", shoffset + 1);  /*CWK*/
      /* diese beiden vorgezogen 950814 CWK. neu: + 1  */
      shoffset = roundup(shoffset + p->type->size, p->type->align);
@@ -1659,6 +1740,15 @@ long strtol(str, ptr, b) char *str, **ptr; {
 
 /* === Funktionen fuer ex-p-code Befehle, CWK-neu 940628: ==================== */
 
+/* shspace(): allocate space for shared block-local variables */ 
+void shspace(int k)      /*970902*/
+{
+    Tree e = tree(SHSPACE+V, voidtype, 0, 0);
+    code(Pcode);
+    codelist->u.node = listnodes( e, 0, 0 );
+    codelist->par = k;
+}
+
 /* splitSM(): erzeuge Code zur Berechnung der Trennzelle,
  * um freien SM-Bereich gleichmaessig aufzuteilen */
 void splitSM()
@@ -1674,6 +1764,7 @@ void nextcase( erster )
    int erster;
 {
    Tree e = tree(NEXTCASE+V, voidtype, 0, 0);
+   shframedepth++;  /*970902*/
    code(Pcode);
    codelist->u.node = listnodes( e, 0, 0 );
    codelist->par = erster;
@@ -1683,6 +1774,7 @@ void nextcase( erster )
 void lgrp_if()
 {
    Tree e = tree(LGRPIF+V, voidtype, 0, 0);
+   shframedepth--;  /*970902*/
    code(Pcode);
    codelist->u.node = listnodes( e, 0, 0 );
 }
@@ -1699,6 +1791,7 @@ void synchronize()
 void engrp()
 {
    Tree e = tree(ENGRP+V, voidtype, 0, 0);
+   shframedepth ++;  /*970902*/
    code(Pcode);
    codelist->u.node = listnodes( e, 0, 0 );
 }
@@ -1707,6 +1800,7 @@ void engrp()
 void exgrp()
 {
    Tree e = tree(EXGRP+V, voidtype, 0, 0);
+   shframedepth --;
    code(Pcode);
    codelist->u.node = listnodes( e, 0, 0 );
 }
@@ -1723,6 +1817,7 @@ void xproc()
 void enlgrp( looptyp )
 {
    Tree e = tree(ENLGRP+V, voidtype, 0, 0);
+   shframedepth ++;
    code(Pcode);
    codelist->u.node = listnodes( e, 0, 0 );
    codelist->par = looptyp;    /*950504*/
@@ -1732,6 +1827,7 @@ void enlgrp( looptyp )
 void exlgrp()
 {
    Tree e = tree(EXLGRP+V, voidtype, 0, 0);
+   shframedepth --;
    code(Pcode);
    codelist->u.node = listnodes( e, 0, 0 );
 }
@@ -1765,17 +1861,32 @@ void flushframes( k )
    code(Pcode);
    codelist->u.node = listnodes( e, 0, 0 );
    codelist->par = k;
+   /*shframedepth -= k;   /*970902*/
+   /*muss weg, weil die shframedepth eine
+     statische und keine dynamische Groesse ist! 980129 */
+   shframedepth --;
+   /*weil der Compiler zu schlau ist
+     und das naechste lgrpif o.dgl. wegoptimiert*/
 }
 
+
 /* retflush():  flush some shared group frames, care about starts CWK */
 void retflush( p )  Node p; {
  int i;
  assert( p->rframes >= 0); assert( p->started >=0 );
- for (i = p->rframes; i > 0; i--) {
+ /*shframedepth -= p->rframes;   /*970902*/
+   /*muss weg, weil die shframedepth eine
+     statische und keine dynamische Groesse ist! 980129 */
+ shframedepth --;
+   /*weil der Compiler zu schlau ist
+     und das naechste lgrpif o.dgl. wegoptimiert*/
+ if (p->rframes > 0)
     force_modulo_0;
+ for (i = p->rframes; i > 0; i--) {
     print("getlo\t -1,r31   \t /*RETFLUSH(%d)*/\n", i);
     print("mpaddn\t gps,1,r31\t /*RETFLUSH(%d)*/\n", i);
     print("ldg\t gps,0,gps\t /*RETFLUSH(%d)*/\n", i);
+    print("mov\t gps,sps\t /*RETFLUSH(%d)*/\n", i);   /*970904*/
  }
  if (p->started) {     /*issue an xproc(): */
          /*Die Gruppenrahmen vor der start-Anweisung werden wiederhergestellt.*/
@@ -1783,9 +1894,11 @@ void retflush( p )  Node p; {
          print("getlo\t -1,r31   \t /*retxproc: aus startGruppe*/\n");
          print("mpadd\t gps,1,r31\t /*retxproc: austragen*/\n");
          /*print("bsrg\t spp,forklib_sync\n"); weg nach Absprache mit HS*/
-         print("ldg\t gps,0,gps\t /*xproc: restore gps*/\n");       /*0*/
-         print("mov\t gpp,spp\t /*xproc: prepare spp*/\n");
-         print("ldg\t gpp,0,gpp\t /*xproc: restore gpp*/\n");       /*0*/
+         print("ldg\t gpp,-3,fps\t /*retxproc: restore fps*/\n"); /*1*980129*/
+         print("ldg\t gps,0,gps\t /*retxproc: restore gps*/\n");       /*0*/
+         print("mov\t gps,sps\t /*retxproc: restore sps*/\n");
+         print("mov\t gpp,spp\t /*retxproc: prepare spp*/\n");
+         print("ldg\t gpp,0,gpp\t /*retxproc: restore gpp*/\n");       /*0*/
 #if 0
 CWK 960812
          print("gethi\t ___PROC_NR__,par2 /*xproc: $orig.*/\n");
@@ -1797,7 +1910,10 @@ CWK 960812
 #endif
          print("popg\t spp,eps\t /*xproc: restore eps*/\n");       /*0*/
          /*Alle Prozessoren ausser 0 setzen das SHADOW-Bit:*/
+#if 0
+    970904 Wieso?? Damit kann ich mir den Prozedurframe zerhacken!
          print("add\t gpp,3,spp\t /*xproc: restore spp*/\n");
+#endif
 #if 0
 CWK 960812
          print("ldg\t par2,0,par2\t /*xproc: $orig.*/\n");           /*0*/
@@ -1816,6 +1932,7 @@ CWK 960812
  }
 }
 
+
 /* begseq(): enter seq body, mask out all but one processor 970310 */
 void begseq( int labelnr ) 
 {
