@@ -170,6 +170,7 @@ static void dostmt(lab, swp, lev, rframes) struct swtch *swp; {
 static void xxxdostmt(lab, swp, lev, rframes) struct swtch *swp; {
    Tree e;  /*CWK*/
    extern void enlgrp(int), size(), exlgrp(), synchronize();  /*CWK*/
+   extern int shframedepth; 
    int priv;   /*CWK*/
    /* CWK:  Da das statement zuerst geemitted wird, aber erst nach Parsen von
     * e bekannt ist, ob private Pcode-Befehle eingefuegt werden muessen,
@@ -206,7 +207,9 @@ static void xxxdostmt(lab, swp, lev, rframes) struct swtch *swp; {
    enlgrp( 2 );
    definelab(lab);
    size();
+   shframedepth++;
    statement(lab, swp, lev, 0, 0, rframes + 1);
+   shframedepth--;
    definelab(lab + 1);
    expect(WHILE);
    expect('(');
@@ -336,6 +339,7 @@ static void xxxforstmt(lab, swp, lev, rframes) struct swtch *swp; {
    static char follow[] = { IF, ID, '}', 0 };
    int priv, reallypriv;     /*CWK*/
    extern void enlgrp(int), exlgrp(), synchronize(), size();
+   extern int shframedepth; 
 
    e1 = e2 = e3 = 0;
    t = gettok();
@@ -368,7 +372,9 @@ static void xxxforstmt(lab, swp, lev, rframes) struct swtch *swp; {
       if (e2)
          walk(e2, 0, lab + 2);
       if (priv && reallypriv) size();    /*CWK, vorher priv */
+      if (priv) shframedepth++;
       statement(lab, swp, lev, 0, 0, rframes + ((priv)? 1:0) ); /*CWK*/
+      if (priv) shframedepth--;
       definelab(lab + 1);
       definept(&pt3);
       if (e3)
@@ -421,7 +427,7 @@ static void forkstmt(lab, swp, lev, bframes, cframes, rframes) struct swtch *swp
    if (kind[t] == ID) {
       e1 = expr(';');
       if (may_dep_on_pr_var( e1 ))  /*CWK*/
-        error("fork: number of groups must not depend on private var!\n");
+         warning("fork: number of groups must not depend on private value!\n");
    }
    else {
       error("fork: number of groups required!\n");
@@ -484,6 +490,10 @@ static void syncstmt(lab, swp, lev) struct swtch *swp; {
 static void startstmt(lab, swp, lev) struct swtch *swp; {
    Tree e;
    extern void sproc(), xproc();
+   extern int shframedepth; 
+   extern int shoffset;
+   int oldshframedepth;
+   int oldshoffset;	
 
    t = gettok();
    if (t=='(')  {  /*Expression follows*/
@@ -512,7 +522,12 @@ static void startstmt(lab, swp, lev) struct swtch *swp; {
      codelist->u.node = listnodes(e, 0, 0);
    }
    isinsyncmode = 1; isinsideastartbody = 1;
+   oldshframedepth = shframedepth; /*980129*/
+   shframedepth = 0; /*vorher ++ 980129*/
+   /*testweise raus 970908 oldshoffset = shoffset; shoffset = 0;   /*970903*/
    statement(lab, swp, lev, 0, 0, 0);   /*CWK: no former frames to flush. */
+   shframedepth = oldshframedepth; /*980129*/
+   /* testweise raus 970908 shoffset = oldshoffset;   /*970903*/
    isinsyncmode = 0; isinsideastartbody = 0;
    xproc();
 }
@@ -546,6 +561,7 @@ static void xxxifstmt(lab, loop, swp, lev, bframes, cframes, rframes) struct swt
    extern void nextcase();   /*CWK*/
    extern void lgrp_if();   /*CWK*/
    extern void synchronize();   /*CWK*/
+   extern int shframedepth; 
    t = gettok();
    expect('(');
    definept(0);
@@ -556,13 +572,17 @@ static void xxxifstmt(lab, loop, swp, lev, bframes, cframes, rframes) struct swt
    if (priv) splitSM();      /*CWK: freien Speicher gleichmaessig aufteilen */
    walk(e, 0, lab);
    if (priv) nextcase(0);    /*CWK: then-Gruppe aufmachen */
+   if (priv) shframedepth++;
    statement(loop, swp, lev, bframes+((priv)?1:0), cframes+((priv)?1:0), rframes + ((priv)?1:0));/*CWK*/
+   if (priv) shframedepth--;
    if (t == ELSE) {
       branch(lab + 1);    /* zum lgrp_if springen */
       t = gettok();
       definelab(lab);
       if (priv) nextcase(1); /*CWK: else-Gruppe aufmachen */
+      if (priv) shframedepth++;
       statement(loop, swp, lev, bframes+((priv)?1:0), cframes+((priv)?1:0), rframes + ((priv)?1:0));/*CWK*/
+      if (priv) shframedepth--;
       if (findlabel(lab + 1)->ref)
          definelab(lab + 1);
    } else {  /*dummy-Else, um die idle Prozessoren zu beschaeftigen: */
@@ -576,7 +596,8 @@ static void xxxifstmt(lab, loop, swp, lev, bframes, cframes, rframes) struct swt
    if (priv) synchronize();  /*CWK: synchr. im alten Frame */
 }
 
-/* localaddr - returns q if p yields the address of local/parameter q; otherwise returns 0 */
+/* localaddr - returns q if p yields the address of local/parameter q;
+   otherwise returns 0 */
 static Symbol localaddr(p) Tree p; {
    if (p == 0)
       return 0;
@@ -584,6 +605,7 @@ static Symbol localaddr(p) Tree p; {
    case INDIR: case CALL: case ARG: case RARG: case SARG:  /*CWK*/
       return 0;
    case ADDRL: case ADDRF:
+   case ADDRSL: case ADDRSF:   /*neu 970902 CWK*/
       return p->u.sym;
    case RIGHT: case ASGN:
       if (p->kids[1])
@@ -1121,6 +1143,7 @@ static void xxxwhilestmt(lab, swp, lev, rframes) struct swtch *swp; {
    Tree e;
    int priv;        /*CWK*/
    extern void enlgrp(int), exlgrp(), size(), synchronize();
+   extern int shframedepth; 
 
    refinc *= 10;
    t = gettok();
@@ -1136,14 +1159,18 @@ static void xxxwhilestmt(lab, swp, lev, rframes) struct swtch *swp; {
       definelab(lab + 1);
       definept(&pt);
       walk(e, 0, lab + 2);
+      if (priv) shframedepth++;
       statement(lab, swp, lev, 0, 0, rframes + ((priv)?1:0)); /*CWK*/
+      if (priv) shframedepth--;
       branch(lab);
    } else {   /* goto L+1; L: S; L+1: if (E) goto L; L+2: */
       if (priv) enlgrp( 0 );    /*CWK*/
       branch(lab + 1);
       definelab(lab);
       if (priv) size();     /*CWK*/
+      if (priv) shframedepth++;
       statement(lab, swp, lev, 0, 0, rframes + ((priv)?1:0)); /*CWK*/
+      if (priv) shframedepth--;
       definelab(lab + 1);
       definept(&pt);
       walk(e, lab, 0);
